/* - RFID full -
 * This example displays a gui that shows the attached Phidgets RFID reader device's details and will
 * display and tag data that is scanned by the antenna.  it also provides checkboxes to manipulate the
 * different attirbutes of the RFID reader as well as the digital outputs.
 *
 * Please note that this example was designed to work with only one Phidget RFID connected.
 * For an example showing how to use two Phidgets of the same time concurrently, please see the
 * Servo-multi example in the Servo Examples.
 *
 * Copyright 2007 Phidgets Inc.
 * This work is licensed under the Creative Commons Attribution 2.5 Canada License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by/2.5/ca/
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace RFID_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
        formCleaner cleaner;
        private RFID rfid; //Declare an RFID object
		private ErrorEventBox errorBox;
		Boolean runonce = false;

		LockWarning warningForm;

		public Form1() {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
			InitializeComponent();
		}

		//initialize our Phidgets RFID reader and hook the event handlers
		private void Form1_Load(object sender, EventArgs e) {
			writeProtoCmb.DataSource = typeof(RFIDProtocol).ToList();

			errorBox = new ErrorEventBox(this);
			rfid = new RFID();

			rfid.Attach += rfid_Attach;
			rfid.Detach += rfid_Detach;
			rfid.Error += rfid_Error;

			rfid.Tag += rfid_Tag;
			rfid.TagLost += rfid_TagLost;

            commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

            try
            { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
                rfid.Channel = phidgetParameters.Channel; //selects the channel on the device to open
                rfid.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
                rfid.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
                rfid.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

                if (phidgetParameters.isRemote) //are we trying to open a remote device?
                {
                    rfid.IsRemote = true;
                    Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
                    if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
                        Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
                }
                else
                    rfid.IsLocal = true;

                rfid.Open(); //open the device specified by the above parameters
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

        }

		//attach event handler..populate the details fields as well as display the attached status.  enable the checkboxes to change
		//the values of the attributes of the RFID reader such as enable or disable the antenna and onboard led.
		void rfid_Attach(object sender, AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);
			readBox.Visible = true;

			RFID attachedDevice = (RFID)sender;

			switch (attachedDevice.DeviceID) {
			case DeviceID.PN_1024:
				writeBox.Visible = true;
				break;
			case DeviceID.PN_1023:
				writeBox.Visible = false;
				break;
			}

			antennaChk.Visible = true;
			keyboardCheckBox.Visible = true;
			antennaChk.Checked = true;
            try {
                rfid.AntennaEnabled = true;
            }
            catch(PhidgetException ex) { errorBox.addMessage("Error enabling antenna: " + ex.Message); }
		}

		//detach event handler...clear all the fields, display the attached status, and disable the checkboxes.
		void rfid_Detach(object sender, DetachEventArgs e) {
			phidgetInfoBox1.Clear();
            cleaner.clean();

            writeBox.Visible = false;
			antennaChk.Visible = false;
			keyboardCheckBox.Visible = false;
			readBox.Visible = false;
			runonce = false;
		}

		void rfid_Error(object sender, ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		//Tag event handler...we'll display the tag code in the field on the GUI
		void rfid_Tag(object sender, RFIDTagEventArgs e) {
			tagTxt.Text = e.Tag;
			protoTxt.Text = e.Protocol.GetDescription();

			//This sends the RFID tag and an enter to the active application
			if (keyboardCheckBox.Checked == true) {
				SendKeys.Send(e.Tag);
				SendKeys.Send("{ENTER}");
			}
		}

		//Tag lost event handler...here we simply want to clear our tag field in the GUI
		void rfid_TagLost(object sender, RFIDTagLostEventArgs e) {
			tagTxt.Text = "";
			protoTxt.Text = "";
		}

		//Enable or disable the RFID antenna by clicking the checkbox
		private void antennaChk_CheckedChanged(object sender, EventArgs e) {
			try {
				rfid.AntennaEnabled = antennaChk.Checked;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error enabling antenna: " + ex.Message);
			}
		}

		//When the application is being terminated, close the Phidget.
		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			rfid.Close();

			if (warningForm != null)
				warningForm.Close();
		}

		private void writeBtn_Click(object sender, EventArgs e) {
			try {
				RFIDProtocol proto = (RFIDProtocol)Enum.Parse(typeof(RFIDProtocol), writeProtoCmb.SelectedValue.ToString());
				rfid.Write(writeTagTxt.Text, proto, writeLockChk.Checked);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error writing tag: " + ex.Message);
			}
		}

		private void writeLockChk_CheckedChanged(object sender, EventArgs e) {
			//let the user know that this will permanently lock the RFID tag making it no longer writable
			if (writeLockChk.Checked == true && runonce == false) {
				warningForm = new LockWarning(this);
				warningForm.ShowDialog();
				runonce = true;
			}
		}

		public Boolean lockCheck {
			get { return writeLockChk.Checked; }
			set { writeLockChk.Checked = value; }
		}
	}
}